"""
Plot memory usage of a numeric computation using numpy and scipy
"""
import pylab as pl
import numpy as np
from memory_profiler import memory_usage
from scipy import linalg

X = np.random.randn(1000, 1000)

mem = memory_usage((linalg.qr, (X,)))
x = np.linspace(0, len(mem) * .1, len(mem))

p = pl.fill_between(x, mem)

pl.xlabel('time')
pl.ylabel('Memory consumption (in MB)')
pl.show()



# -*- coding: utf-8  -*-

from memory_profiler import profile

f = open('/dev/null', 'w')
@profile(stream=f)
def test_unicode(txt):
    # test when unicode is present
    txt = txt.replace (u"ی", u"ي") #Arabic Yah = ي
    return txt


if __name__ == '__main__':
	test_unicode (u"ایست")



# .. an example with a for loop ..

import time

@profile
def test_1():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    del b

    def test_2():
        a = [1] * (10 ** 6)
        b = [2] * (2 * 10 ** 7)
        del b

        return a

    return test_2


if __name__ == '__main__':
    profile.enable_by_count()

    test_2 = test_1()
    time.sleep(1)
    test_2()
    time.sleep(1)

    profile.disable_by_count()



from memory_profiler import profile


@profile
def test_with_profile(arg1):
    """dummy doc"""
    return None


if __name__ == '__main__':
    assert test_with_profile.__doc__ == "dummy doc"
    assert test_with_profile.__name__ == "test_with_profile"



import asyncio

from memory_profiler import profile


@profile
@asyncio.coroutine
def foo():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    yield from asyncio.sleep(1)
    del b
    return a


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(foo())



from memory_profiler import memory_usage
import os


def some_func(*args, **kwargs):
    return args, kwargs


def test_memory_usage():
    # Check that memory_usage works with functions with star args.
    mem, ret = memory_usage((some_func, (1, 2), dict(a=1)), retval=True)
    assert ret[0] == (1, 2)
    assert ret[1] == dict(a=1)
    
    
def write_line(filepath):
    with open(filepath, 'a') as the_file:
        the_file.write('Testing\n')

def test_max_iterations():
    # Check that memory_usage works with max_iterations set (for python functions).
    this_dir = os.path.dirname(os.path.realpath(__file__))
    file = os.path.join(this_dir, 'temp_test_max_iterations_file.txt')
    mem = memory_usage((write_line, (file, ), dict()), max_usage=True, max_iterations=1)
    n_lines = sum(1 for line in open(file))
    os.remove(file)
    assert n_lines == 1


def test_return_value_consistency():
    # Test return values when watching process by PID
    pid_mem_list = memory_usage(timeout=1)
    assert type(pid_mem_list) == list, "Memory usage of process should be a list"
    pid_mem_max = memory_usage(timeout=1, max_usage=True)
    assert type(pid_mem_max) == float, "Max memory usage of process should be a number"
    # Test return values when watching callable
    func_mem_list = memory_usage((some_func, (42,), dict(a=42)))
    assert type(func_mem_list) == list, "Memory usage of callable should be a list"
    func_mem_max = memory_usage((some_func, (42,), dict(a=42)), max_usage=True)
    assert type(func_mem_max) == float, "Max memory usage of callable should be a number"


if __name__ == "__main__":
    test_memory_usage()
    test_max_iterations()
    test_return_value_consistency()



from memory_profiler import profile

f=open('hi.txt','w+')

@profile(stream=f)
def my_func():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    del b
    return a

@profile(stream=f)
def my_func1():
    a = [2] * (10 ** 6)
    b = [3] * (2 * 10 ** 7)
    del b
    return a

if __name__ == '__main__':
    my_func()
    my_func1()


# -*- coding: utf-8  -*-
@profile
def test_unicode(txt):
    # test when unicode is present
    txt = txt.replace (u"ی", u"ي") #Arabic Yah = ي
    return txt
if __name__ == '__main__':
	test_unicode (u"ایست")


from memory_profiler import profile
import logging

# create logger
logger = logging.getLogger('memory_profile_log')
logger.setLevel(logging.DEBUG)

# create file handler which logs even debug messages
fh = logging.FileHandler("memory_profile.log")
fh.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(formatter)

# add the handlers to the logger
logger.addHandler(fh)

from memory_profiler import LogFile
import sys
sys.stdout = LogFile('memory_profile_log', reportIncrementFlag=False)

@profile
def my_func():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    del b
    return a

@profile
def my_func1():
    a = [2] * (10 ** 6)
    b = [3] * (2 * 10 ** 7)
    del b
    return a

if __name__ == '__main__':
    my_func()
    my_func1()



import unittest
import sys
import tempfile


class TestExitCode(unittest.TestCase):

    def setUp(self):
        # to be able to import mprof
        sys.path.append('.')
        from mprof import run_action
        self.run_action = run_action

    def test_exit_code_success(self):
        s = "1+1"
        tmpfile = tempfile.NamedTemporaryFile('w', suffix='.py')
        with tmpfile as ofile:
            ofile.write(s)
            ofile.flush()
            sys.argv = ['<ignored>', '--exit-code', tmpfile.name]
            self.assertRaisesRegexp(SystemExit, '0', self.run_action)

    def test_exit_code_fail(self):
        s = "raise RuntimeError('I am not working nicely')"
        tmpfile = tempfile.NamedTemporaryFile('w', suffix='.py')
        with tmpfile as ofile:
            ofile.write(s)
            ofile.flush()
            sys.argv = ['<ignored>', '--exit-code', tmpfile.name]
            self.assertRaisesRegexp(SystemExit, '1', self.run_action)

    def test_no_exit_code_success(self):
        s = "raise RuntimeError('I am not working nicely')"
        tmpfile = tempfile.NamedTemporaryFile('w', suffix='.py')
        with tmpfile as ofile:
            ofile.write(s)
            ofile.flush()
            sys.argv = ['<ignored>', tmpfile.name]
            self.run_action()

if __name__ == '__main__':
    unittest.main()



from io import StringIO
from time import sleep

from memory_profiler import profile

try:
    import tracemalloc
    has_tracemalloc = True
except ImportError:
    has_tracemalloc = False

output = StringIO()

# allowable error in MB
EPSILON = 0.0001


def test_memory_profiler(test_input, expected):
    mem_prof(test_input)
    inc, dec = parse_mem_prof()
    assert abs(inc - expected) <= EPSILON, \
        'inc = {}, size = {}, err = {}'.format(
            inc, expected, abs(inc - expected)
        )


@profile(stream=output, precision=6, backend='tracemalloc')
def mem_prof(n):
    a = bytearray(n)
    del a
    sleep(1)


def parse_mem_prof():
    text = output.getvalue().split('\n')

    def f(s):
        return float(s.split()[3])

    return f(text[-6]), -f(text[-5])

if __name__ == '__main__':
    if has_tracemalloc:
        tests = [
            (100, 0.00012302398681640625),
            (1000, 0.0009813308715820312),
            (10000, 0.009564399719238281),
            (100000, 0.09539508819580078),
            (1000000, 0.9537019729614258),
            (10000000, 9.536770820617676),
            (100000000, 95.36745929718018),
        ]
        for test_input, expected in tests:
            test_memory_profiler(test_input, expected)



from memory_profiler import memory_usage

# size = 50000
size = 3000


def test_simple():

    import numpy as np

    def func():
        a = np.random.random((size, size))
        return a

    rss = memory_usage(proc=func, max_usage=True, backend="psutil")
    uss = memory_usage(proc=func, max_usage=True, backend="psutil_uss")
    pss = memory_usage(proc=func, max_usage=True, backend="psutil_pss")
    print(rss, uss, pss)


def test_multiprocessing():

    import numpy as np
    import joblib
    import time

    def func():
        n_jobs = 4
        a = np.random.random((size, size))

        def subprocess(i):
            time.sleep(2)
            return a[i,i]

        results = joblib.Parallel(n_jobs=n_jobs)(
            joblib.delayed(subprocess)(i) 
            for i in range(n_jobs))

        return results

    rss = memory_usage(proc=func, max_usage=True, backend="psutil", include_children=True, multiprocess=True)
    uss = memory_usage(proc=func, max_usage=True, backend="psutil_uss", include_children=True, multiprocess=True)
    pss = memory_usage(proc=func, max_usage=True, backend="psutil_pss", include_children=True, multiprocess=True)
    print(rss, uss, pss)


def test_multiprocessing_write():

    import numpy as np
    import joblib
    import time

    def func():
        n_jobs = 4
        a = np.random.random((size, size))

        def subprocess(i):
            aa = a.copy()
            time.sleep(2)
            return aa[i,i]

        results = joblib.Parallel(n_jobs=n_jobs)(
            joblib.delayed(subprocess)(i) 
            for i in range(n_jobs))

        return results

    rss = memory_usage(proc=func, max_usage=True, backend="psutil", include_children=True, multiprocess=True)
    uss = memory_usage(proc=func, max_usage=True, backend="psutil_uss", include_children=True, multiprocess=True)
    pss = memory_usage(proc=func, max_usage=True, backend="psutil_pss", include_children=True, multiprocess=True)
    print(rss, uss, pss)


def test_multiprocessing_showcase():

    import numpy as np
    import joblib
    import time
    import datetime

    def func():

        # n_jobs = 32
        # size = 25000
        # Creating data: 25000x25000 ... done (4.66 Gb). Starting processing: n_jobs=32 ... done (0:00:37.581291). RSS: 353024.01
        # Creating data: 25000x25000 ... done (4.66 Gb). Starting processing: n_jobs=32 ... done (0:00:38.867385). USS: 148608.62
        # Creating data: 25000x25000 ... done (4.66 Gb). Starting processing: n_jobs=32 ... done (0:00:29.049754). PSS: 169253.91

        # n_jobs = 64
        # size = 10000
        # Creating data: 10000x10000 ... done (0.75 Gb). Starting processing: n_jobs=64 ... done (0:00:14.701243). RSS: 111362.79
        # Creating data: 10000x10000 ... done (0.75 Gb). Starting processing: n_jobs=64 ... done (0:00:15.020202). USS: 56108.69
        # Creating data: 10000x10000 ... done (0.75 Gb). Starting processing: n_jobs=64 ... done (0:00:15.072918). PSS: 54826.61
        
        # Conclusion:
        # * RSS is overestimating like crazy (I checked the actual memory usage using htop)

        n_jobs = 8
        size = 3000

        print("Creating data: {size}x{size} ... ".format(size=size), end="")
        a = np.random.random((size, size))
        print("done ({size:.02f} Gb). ".format(size=a.size * a.itemsize / 1024**3), end="")

        def subprocess(i):
            aa = a.copy()
            r = aa[1,1]
            aa = a.copy()
            time.sleep(10)
            return r
            
            # r = a[1,1]
            # # time.sleep(10)
            # return r
            
            pass

        start = datetime.datetime.now()
        print("Starting processing: n_jobs={n_jobs} ... ".format(n_jobs=n_jobs), end="")
        results = joblib.Parallel(n_jobs=n_jobs)(
            joblib.delayed(subprocess)(i) 
            for i in range(n_jobs))
        print("done ({}). ".format(datetime.datetime.now() - start), end="")

        return results

    rss = memory_usage(proc=func, max_usage=True, backend="psutil", include_children=True, multiprocess=True)
    print("RSS: {rss:.02f}".format(rss=rss))
    uss = memory_usage(proc=func, max_usage=True, backend="psutil_uss", include_children=True, multiprocess=True)
    print("USS: {uss:.02f}".format(uss=uss))
    pss = memory_usage(proc=func, max_usage=True, backend="psutil_pss", include_children=True, multiprocess=True)
    print("PSS: {pss:.02f}".format(pss=pss))


if __name__ == "__main__":
    test_simple()
    test_multiprocessing()
    test_multiprocessing_write()
    test_multiprocessing_showcase()



import asyncio
import sys

from memory_profiler import profile


@profile
async def my_func():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    await asyncio.sleep(1e-2)
    del b

async def main():
    task = asyncio.create_task(my_func())
    res = await asyncio.gather(task)

async def main_legacy():
    future = asyncio.ensure_future(my_func())
    res = await asyncio.gather(future)

if __name__ == '__main__':
    if sys.version_info >= (3, 7):
        asyncio.run(main())  # main loop
    else:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(main_legacy())



import unittest

from memory_profiler import LineProfiler, profile, show_results
from io import StringIO


class TestIncrementDisplay(unittest.TestCase):
    """Tests memory incrementation / decrementation display"""

    def test_loop_count(self):

        def some_loop():
            for i in range(12):  # line -2
                a = 1            # line -1

        profiler = LineProfiler()
        wrapped = profiler(some_loop)
        wrapped()
        show_results(profiler)
        for_line = list(list(profiler.code_map.values())[0].values())[-2]
        looped_instruction = list(list(profiler.code_map.values())[0].values())[-1]

        self.assertEqual(for_line[2], 13)
        self.assertEqual(looped_instruction[2], 12)

    def test_normal_incr(self):

        def normal_incr():
            use_some_memory = [1] * (10 ** 6)

        profiler = LineProfiler()
        wrapped = profiler(normal_incr)
        wrapped()

        show_results(profiler)
        results = list(list(profiler.code_map.values())[0].values())[-1]

        self.assertGreater(results[0], 0)
        self.assertGreater(results[1], results[0])
        self.assertEqual(results[2], 1)

    def test_loop_incr(self):

        def loop_incr():
            a = []
            b = [2] * (2 * 10 ** 7)      # line -4
            for i in range(3):
                c = [2] * (2 * 10 ** 7)  # line -2
                a.append(c)

        profiler = LineProfiler()
        wrapped = profiler(loop_incr)
        wrapped()

        show_results(profiler)
        b_line = list(list(profiler.code_map.values())[0].values())[-4]
        c_line = list(list(profiler.code_map.values())[0].values())[-2]
        self.assertAlmostEqual(b_line[2] * 3, c_line[2], delta=1)
        self.assertEqual(c_line[2], 3)

    def test_decr(self):

        def del_stuff():
            b = [2] * (2 * 10 ** 7)
            del b

        profiler = LineProfiler()
        wrapped = profiler(del_stuff)
        wrapped()

        show_results(profiler)
        b_line = list(list(profiler.code_map.values())[0].values())[-2]
        del_line = list(list(profiler.code_map.values())[0].values())[-1]

        self.assertGreater(0, del_line[0])
        self.assertGreater(del_line[1], 0)
        self.assertAlmostEqual(-del_line[0], b_line[0], delta=1)


if __name__ == '__main__':
    unittest.main()



import numpy as np
import scipy.signal


@profile
def create_data():
    ret = []
    for n in range(70):
        ret.append(np.random.randn(1, 70, 71, 72))
    return ret


@profile
def process_data(data):
    data = np.concatenate(data)
    detrended = scipy.signal.detrend(data, axis=0)
    return detrended


if __name__ == "__main__":
    data1 = create_data()
    data2 = process_data(data1)



"""
An undecorated example of a script that allocates memory in multiprocessing
workers to demonstrate the use of memory_profiler with multiple processes.

Run this script with mprof run -M multiprocessing_example.py
You can then visualize the usage with mprof plot.
"""

import time
import multiprocessing as mp

# Big numbers
X6 = 10 ** 6
X7 = 10 ** 7


def worker(num, wait, amt=X6):
    """
    A function that allocates memory over time.
    """
    frame = []

    for idx in range(num):
        frame.extend([1] * amt)
        time.sleep(wait)

    del frame


def main_sequential():
    """
    A sequential version of the work, where one worker is called at a time.
    """
    worker(5, 5, X6)
    worker(5, 2, X7)
    worker(5, 5, X6)
    worker(5, 2, X7)


def main_multiproc():
    """
    A multiprocessing version of the work, where workers work in their own
    child processes and are collected by the master process.
    """
    pool    = mp.Pool(processes=4)
    tasks   = [
        pool.apply_async(worker, args) for args in
        [(5, 5, X6), (5, 2, X7), (5, 5, X6), (5, 2, X7)]
    ]

    results = [p.get() for p in tasks]


if __name__ == '__main__':
    main_multiproc()



"""This script is intended as a test case for mprofile"""

import time

@profile
def test1(l):
    """test1 docstring"""
    a = [1] * l
    time.sleep(1)
    return a

@profile
def test2(l):
    b = [1] * l
    time.sleep(1)
    return b

def test3(l):
    """test3 docstring"""
    return l

if __name__ == "__main__":
    l = 100000
    test1(l)
    test2(2 * l)

    # make sure that the function name and docstring are set
    # by functools.wraps
    # memory_profile.py def profile func is not None case
    assert (test1.__name__ == 'test1'), 'function name is incorrect'
    assert (test1.__doc__ == 'test1 docstring'), 'function docstring is incorrect'
    # memory_profile.py def profile func is None case
    profile_maker = profile()
    profiled_test3 = profile_maker(test3)
    assert (profiled_test3.__name__ == 'test3'), 'function name is incorrect'
    assert (profiled_test3.__doc__ == 'test3 docstring'), 'function docstring is incorrect'




@profile
def my_func():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    del b
    yield a


@profile
def test_comprehension():
    # Dict comprehension
    d_comp = dict((str(k*k), [v] * (1<<17))
                  for (v, k) in enumerate(range(99, 111)))

    # List comprehension
    l_comp = [[i] * (i<<9) for i in range(99)]
    del l_comp
    del d_comp

    def hh(x=1):
        # Set comprehension
        s_comp = set(('Z',) * (k<<13) for k in range(x, 19 + 2*x))
        return s_comp

    val = [range(1, 4), max(1, 4), 42 + len(hh())]
    val = hh() | hh(4)
    val.add(40)
    l1_comp = [[(1, i)] * (i<<9) for i in range(99)]
    l2_comp = [[(3, i)] * (i<<9) for i in range(99)]

    return val


@profile
def test_generator():
    a_gen = ([42] * (1<<20) for __ in '123')
    huge_lst = list(a_gen)

    b_gen = ([24] * (1<<20) for __ in '123')
    del b_gen
    del huge_lst

    return a_gen


if __name__ == '__main__':
    with profile:
        next(my_func())     # Issue #42

    test_generator()
    test_comprehension()



# .. an example with a for loop ..

import time

def test_1():
    a = {}
    for i in range(10000):
        a[i] =  i + 1
    return

@profile
def test_2():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    del b

    for i in range(2):
        a = [1] * (10 ** 6)
        b = [2] * (2 * 10 ** 7)
        del b
    return a


if __name__ == '__main__':
    test_1()
    time.sleep(1)
    test_2()
    time.sleep(1)




# make sure that memory profiler does not hang on exception
from memory_profiler import memory_usage

def foo():
    raise NotImplementedError('Error')

try:
    out = memory_usage((foo, tuple(), {}), timeout=1)
except NotImplementedError:
    pass
print('Success')



# .. an example to test decorator with precision keyword argument ..
"""
Test profiling using command line ``profile`` function (either
``TimeStamper`` or ``LineProfiler``) with precision specified as command
line parameter.
The precision argument specified in the decorator, therefore, is not used.
"""

import time

@profile(precision=4)
def test_1():
    a = [1] * (10 ** 6)
    b = [2] * (2 * 10 ** 7)
    time.sleep(0.6)
    del b

    for i in range(2):
        a = [1] * (10 ** 6)
        b = [2] * (2 * 10 ** 7)
        del b
    return a

@profile(precision=2)
def test_2():
    a = {}
    time.sleep(0.5)
    for i in range(10000):
        a[i] = i + 1
    time.sleep(0.6)
    return

if __name__ == '__main__':
    test_1()
    test_2()



import unittest

import mprof

class Test_function_labels(unittest.TestCase):
    def test(self):
        expected = {
            "x.z": "z",
            "x.y": "y",
            "x.b": "x.b",
            "f.a.b": "f.a.b",
            "g.a.b": "g.a.b",
            "g.a.c": "a.c",
            "b.c": "b.c",
        }
        result = mprof.function_labels(expected.keys())
        self.assertEqual(expected,result)

if __name__ == "__main__":
    unittest.main()
